package com.parhamdata.translator.controllers;

import com.parhamdata.translator.model.Translator;
import com.parhamdata.translator.services.DatabaseService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import java.util.Optional;

@RestController
public class RestAPI {

  private static final Logger log = LoggerFactory.getLogger(RestAPI.class);

  @Autowired
  DatabaseService dbService;

  @PostMapping("/translator")
  Translator create(@RequestBody Translator payload) {
    log.info("Adding Translator Entry for '" + payload.getInput() + "'.");
    Translator result = dbService.save(payload);
    log.info("Added Record: " + result);
    return result;
  }

  @GetMapping("/translator")
  Iterable<Translator> read() {
    log.info("Requesting Translation Table.");
    Iterable<Translator> results = dbService.findAll();
    log.info("Found " + String.valueOf(dbService.count()) + " Records." );
    return results;
  }

  @GetMapping("/translator/id/{id}")
  Optional<Translator> getRecord(@PathVariable Integer id) {
    log.info("Requesting Translator Record [" + String.valueOf(id) + "].");
    Optional<Translator> result =  dbService.findById(id);
    log.info("Result Returned: " + result);
    return result;
  }

  @GetMapping("/translator/{input}")
  Optional<Translator> getRecord(@PathVariable String input) {
    log.info("Requesting Translator Record for '" + input + "'.");
    Optional<Translator> result = dbService.findByInput(input);
    log.info("Result Returned: " + result);
    return result;
  }

  @PutMapping("/translator")
  Translator update(@RequestBody Translator payload) {
    log.info("Updating Translator Record for '" + payload.getInput() + "'.");
    Translator result = dbService.save(payload);
    log.info("Updated Record: " + result);
    return result;
  }

  @DeleteMapping("/translator/delete/{id}")
  ResponseEntity<String> delete(@PathVariable Integer id) {
    log.info("Deleting Translator Record [" + String.valueOf(id) + "].");
    dbService.deleteById(id);
    return ResponseEntity.ok("{}");
  }
}
